library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

-- Module to communicate with LTC1407
-- can be used for single-shot conversion (both channels) or 
-- free running conversion.
-- At free running conversion, memory address lines will be incremented on each sample
entity sr_adc2 is
    Port ( cs      : in    STD_LOGIC;   -- start with setting cs high
           ss      : in    STD_LOGIC;   -- single shot (high) or free running (low)
           i_clock : in    STD_LOGIC;   -- system clock
           o_clock : inout STD_LOGIC;   -- clock line to ADC, will be internal clock too
           aconv   : inout STD_LOGIC;   -- trigger to start conversion
           adata   : in    STD_LOGIC;   -- serial data from ADC
           nWE     : out   STD_LOGIC;   -- negated write enable (memory)
           data    : out   STD_LOGIC_VECTOR (15 downto 0);   -- result data from ADC
           addr    : out   STD_LOGIC_VECTOR (20 downto 0));  -- self incrementing memory address in free running mode
end sr_adc2;

architecture std of sr_adc2 is
signal   lpm_sr         : STD_LOGIC_VECTOR(15 downto 0);
signal   lpm_cnt        : STD_LOGIC_VECTOR(20 downto 0);
shared variable run     : bit       := '1';
shared variable first   : bit       := '1';
shared variable valid   : bit       := '0';
shared variable counter : integer   := 0;
begin
   -- generate serial and internal clock
   process (i_clock, cs) begin
      if (cs = '0') then
         o_clock <= '0';
      elsif rising_edge(i_clock) then
         o_clock <= not o_clock;
      end if;
   end process;

   -- generated conversion start trigger at cs getting high
   process (o_clock, cs) begin
      if (cs = '0') then
         aconv <= '0';
         first := '1';
      elsif rising_edge(o_clock) then
         if (run = '1') then
            if (first = '1') then
               aconv <= '1';
               first := '0';
            else   
               aconv <= '0';
            end if;   
         end if;   
      end if;
   end process;

   -- create virtual counter and process it
   process (o_clock) begin
      if rising_edge(o_clock) then
         if (aconv = '1') then
            counter := 0;
            nWE     <= '1';
         else   
            counter := counter + 1;
         end if;
         if (counter = 2 or counter = 18) then
            lpm_sr <= (others=>'0');
            valid  := '0';
         elsif (counter > 3 and counter < 17) or (counter > 19 and counter < 33) then
            lpm_sr <= lpm_sr(14 downto 0) & adata;    
         elsif (counter = 15 or counter = 31) then
            nWE <= '1';
         elsif (counter = 17 or counter = 33) then
            data    <= lpm_sr;
            lpm_cnt <= std_logic_vector(unsigned(lpm_cnt) + 1);
            addr    <= lpm_cnt;
            valid   := '1';
         elsif (counter = 18 or counter = 34) then
            if (valid = '1') then
               nWE <= '0';
            end if;   
         elsif (counter = 50) then   
            if (ss = '0') then
               run := '1';
            else
               run := '0';
            end if; 
         end if;
      end if;
   end process;
end std;
